function loadTiledMap(mapname)
  local map = love.filesystem.load("res/maps/" .. mapname .. ".lua")()
  map.quads = {}
  map.walls = {}
  map.floor = {}

  local tileset = res.maps.tileset

  map.tileset = tileset
  map.image = res.images.tileset

  for y = 0, (tileset.imageheight / tileset.tileheight) - 1 do
    for x = 0, (tileset.imagewidth / tileset.tilewidth) - 1 do
      local quad = love.graphics.newQuad(
        x * tileset.tilewidth,
        y * tileset.tileheight,
        tileset.tilewidth,
        tileset.tileheight+8,
        tileset.imagewidth,
        tileset.imageheight
      )
      table.insert(map.quads, quad)
    end
  end

  function map:itox(i,w)
    return (i-1) % w + 1
  end

  function map:itoy(i,w)
    return (i-self:itox(i,w)) / w + 1
  end

  function map:draw()
    for i, layer in ipairs(self.layers) do
      for y = 0, layer.height - 1 do
        for x = 0, layer.width - 1 do
          local index = (x + y * layer.width) + 1
          local tid = layer.data[index]

          if tid ~= 0 then
            local quad = self.quads[tid]
            local xx = x * self.tileset.tilewidth
            local yy = y * self.tileset.tileheight
            if layer.name == "walls" then yy = yy - 8 end -- lift walls

            love.graphics.draw(
              self.image,
              quad,
              xx,
              yy
            )
          end
        end
      end
    end
  end

  function map:update()
    for i, tile in pairs(self.floor) do
      if tile.stability > 0 then
        tile.stability = tile.stability - 1
        if tile.stability == 0 then
          map.layers[1].data[tile.index] = 0
          tile.isWall = true
        end
      end
    end
  end

  function map:setupColliders(world)
    for _,layer in ipairs(self.layers) do
      if layer.name == "walls" then
        for i = 1, #layer.data do
          if layer.data[i] ~= 0 then
            self.walls[#self.walls+1] = {x = self:itox(i, layer.width)*16 - 16,
            y = self:itoy(i, layer.width)*16 - 16, isWall = true, draw = function(self) love.graphics.draw(res.images.tileset, map.quads[1], self.x, self.y-8) end}
            local wall = self.walls[#self.walls]
            world:add(wall, wall.x, wall.y, 16, 16)
            table.insert(actors, wall)
          end
        end
      elseif layer.name == "ground" then
        for i = 1, #layer.data do
          if layer.data[i] ~= 0 then
            self.floor[#self.floor+1] = {x = self:itox(i, layer.width)*16 - 16,
            y = self:itoy(i, layer.width)*16 - 16, index = i, stability = 0}
            local tile = self.floor[#self.floor]
            world:add(tile, tile.x, tile.y, 16, 16)
          end
        end
      end
    end
  end

  function map:boxDraw()
    for i, box in ipairs(self.floor) do
      love.graphics.rectangle("line", box.x, box.y, 16, 16)
    end
    love.graphics.setColor(1,0,0)
    for i, box in ipairs(self.walls) do
      love.graphics.rectangle("line", box.x, box.y, 16, 16)
    end
    love.graphics.setColor(1,1,1)
  end

  return map
end
